/*
 * Proiect PM 2021
 * POV display
 * Carapcea Andrei-Antonio 336CA
 */
#define CNT_OVERFLOW 65535

#define NO_SEGMENTS 64

// input capture pin connected to digital output of hall sensor
#define IC_PIN 8

/*
 * POV display configuration.
 * One rotation is divided into 64 angles
 * At angle i, config[i] is displayed by the LEDs
 * The 8 LEDs are connected to PORTD.
 * LSB = external LED
 * MSB = internal LED 
 */
volatile uint8_t config[NO_SEGMENTS] = {
    B11111111,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B11110001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
    B10000001,
};

// current index in display configuration
volatile uint32_t curr_idx = 0;

// Timer1 counter necessary for 1/64 of a rotation
volatile uint32_t segment_cnt = 0;

// number of overflows done by Timer1 while counting a rotation period
volatile uint8_t no_overflows = 0;

void config_timer1()
{
  // normal wave generation
  TCCR1A = 0;
  TCCR1B = 0;

  TCNT1 = 0;
  OCR1A = 0xFFFE;

  // ICNC1 = input capture noise canceller
  TCCR1B |= (1 << ICNC1);

  // CS10 = prescaler 1
  TCCR1B |= (1 << CS10);

  // ICES1 = input capture edge select, 0 = falling edge, 1 = rising edge
  // TCCR1B &= (0 << ICES1);

  // input capture interrupt enable
  TIMSK1 = (1 << ICIE1);

  // enable output compare A match interrupt
  TIMSK1 |= (1 << OCIE1A);

  // timer overflow interrupt enable
  TIMSK1 |= (1 << TOIE1);
}

void setup()
{
  // disable interrupts
  cli();

  // all pins on port D are outputs
  DDRD = 0b11111111;

  // initially turn LEDs off
  PORTD = 0;

  // set IC_pin as input
  DDRB &= (0 << DDB0);

  // enable pullup
  PORTB |= (1 << PORTB0);

  config_timer1();

  // enable interrupts
  sei();
}

void loop()
{
  // do nothing
}

// Interrupt overflow Timer1
ISR(TIMER1_OVF_vect)
{
  no_overflows++;
}

// Interrupt capture Timer1
ISR(TIMER1_CAPT_vect)
{
  // calculate the counter necessary for 1/64 of a rotation
  segment_cnt = (ICR1 + no_overflows * CNT_OVERFLOW) / NO_SEGMENTS;

  // reset display index
  curr_idx = 0;

  // reset timer
  TCNT1 = 0;

  // set OCR1A interrupt threshold
  OCR1A = segment_cnt - 1;

  // reset no_overflows
  no_overflows = 0;
}

// Interrupt compare OCR1A Timer1
ISR(TIMER1_COMPA_vect)
{
  // if out of bounds
  if (curr_idx >= 64)
  {
    //PORTD = 0;
  }
  else
  {
    // display current LED configuration
    PORTD = config[curr_idx];
  }

  // go to the next configuration
  curr_idx++;

  // update OCR1A to trigger for the next segment
  OCR1A = TCNT1 + segment_cnt - 1;
}